import java.util.ArrayList;
import java.util.List;

/**
 * a class representing a paired end read
 * If the read is not paired end, only the first path will be used.
 * @author morani
 *
 */
public class PairPath {

	private List<List<Integer>> _paths;
	private boolean _isCircular;
	
	
	public PairPath(List<Integer> path1,List<Integer> path2) {
		_paths = new ArrayList<List<Integer>>();
		_paths.add(path1);
		_paths.add(path2);
		_isCircular = false;
	}

	public PairPath(List<Integer> path1) {
		_paths = new ArrayList<List<Integer>>();
		_paths.add(path1);
		_paths.add(new ArrayList<Integer>()); //path2
		_isCircular = false;
	}
	public PairPath() {
		_paths = new ArrayList<List<Integer>>();
		_paths.add(new ArrayList<Integer>()); //path1
		_paths.add(new ArrayList<Integer>()); //path2
		_isCircular = false;
	}


	@Override
	public String toString() {
		String res = "PairPath [_paths=" + _paths + "]";
		if (_isCircular)
			res = res+" (C)";
		return res;
			
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_paths == null) ? 0 : _paths.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		PairPath other = (PairPath) obj;
		if (_paths == null) {
			if (other._paths != null)
				return false;
		} else if (!_paths.equals(other._paths))
			return false;
		return true;
	}

	/**
	 * @return the _isCircular
	 */
	public boolean isCircular() {
		return _isCircular;
	}

	/**
	 * @param isCircular the _isCircular to set
	 */
	public void setCircular() {
		_isCircular = true;
	}

	public List<List<Integer>> get_paths() {
		return _paths;
	}

	public List<Integer> getPath1() {
		return _paths.get(0);
	}
	
	public List<Integer> getPath2() {
		return _paths.get(1);
	}

	public void setPath1(List<Integer> path1) {
		_paths.get(0).addAll(path1);
	}
	
	public void setPath2(List<Integer> path2) {
		_paths.get(1).addAll(path2);
	}

	public Integer getFirstID(){
		return (isEmpty())? -10 : _paths.get(0).get(0);
	}

	public boolean isEmpty() {
		return _paths.get(0).isEmpty();
	}

	public void addToPath1(List<Integer> path) {
		_paths.get(0).addAll(path);		
	}

	public void addToPath1(Integer id) {
		_paths.get(0).add(id);
		
	}

	public boolean containsID(Integer i) {
		return ( (!_paths.get(0).isEmpty() && _paths.get(0).contains(i)) || 
				(!_paths.get(1).isEmpty() && _paths.get(1).contains(i)));
	}


	/**
	 * return the max number of occurrences of vid in EITHER path1 or path2
	 * @param vid
	 * @return
	 */
	public int numOccurrences(int vid) {
		
		return Math.max(numOccurrencesSinglePath(0,vid),numOccurrencesSinglePath(1,vid));
		
	}

	
	private int numOccurrencesSinglePath(int i, int vid) {
		int res = 0;
		if (!_paths.get(i).isEmpty())
		{
			for (Integer v :  _paths.get(i))
			{
				if (v.intValue() == vid)
					res++;
			}
		}

		return res;
	}

	public Integer getLastID() {
		return (!hasSecondPath())? getLastID_path1() : _paths.get(1).get(_paths.get(1).size()-1);
	}

	public Integer getFirstID_path2() {
		return (hasSecondPath())? _paths.get(1).get(0) : -10;
	}

	public Integer getLastID_path1() {
		return (isEmpty())? -10 : _paths.get(0).get(_paths.get(0).size()-1);
	}

	public boolean hasSecondPath() {
		return !_paths.get(1).isEmpty();
	}

	public void movePath2To1() {
		_paths.get(0).clear();
		for (Integer v :  _paths.get(1))
			_paths.get(0).add(v);
		
		getPath2().clear();
		
	}


}
